package com.biganto.visual.roompark.base

import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import androidx.appcompat.widget.Toolbar
import androidx.coordinatorlayout.widget.CoordinatorLayout
import androidx.core.view.isGone
import butterknife.BindView
import butterknife.ButterKnife
import com.biganto.visual.roompark.BuildConfig
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.presentation.screen.splash.SplashScreenController
import com.biganto.visual.roompark.util.extensions.setGone
import com.biganto.visual.roompark.util.view_utils.app_bar.DragControlAppBarLayoutBehaviour
import com.bluelinelabs.conductor.Conductor
import com.bluelinelabs.conductor.Router
import com.bluelinelabs.conductor.RouterTransaction
import com.crashlytics.android.Crashlytics
import com.google.android.material.appbar.AppBarLayout
import com.google.android.material.bottomnavigation.BottomNavigationView
import com.google.android.material.textview.MaterialTextView
import io.fabric.sdk.android.Fabric
import kotlinx.android.synthetic.main.activity_main.*
import kotlinx.android.synthetic.main.status_layout_toolbar.view.*
import kotlinx.android.synthetic.main.switch_toolbar.view.*
import timber.log.Timber


class RoomParkMainActivity(
) : BaseRoomParkActivity()
    ,ICollapsingToolBar
    ,IConductorActivity
    ,IBottomNavigation{

//    @Inject
//    lateinit var snackbarProvider: ISnackBarProvider

    private lateinit var router: Router

    @BindView(R.id.top_toolbar) override lateinit var topAppBar: Toolbar
    @BindView(R.id.app_bar) override lateinit var appBar: AppBarLayout
    @BindView(R.id.topToolbarHolder) override lateinit var coordinatorLayout: CoordinatorLayout
    @BindView(R.id.conductor_container) override lateinit var conductorContainer: ViewGroup
    @BindView(R.id.bottom_navigation_view) override lateinit var bottomNavigation: BottomNavigationView
    @BindView(R.id.custom_toolbar_container) lateinit var tbContainer: LinearLayout


    @BindView(R.id.bottom_view_divider) lateinit var bottomShadow: View

    override fun onCreate(savedInstanceState: Bundle?) {
        setTheme(R.style.AppTheme)
        super.onCreate(savedInstanceState)

        if (!BuildConfig.DEBUG) Fabric.with(this, Crashlytics())

        setContentView(R.layout.activity_main)

        ButterKnife.bind(this)

        setSupportActionBar(topAppBar)

        hideAll()

        router = Conductor.attachRouter(this, conductorContainer, savedInstanceState)

        router.setRoot(RouterTransaction.with(SplashScreenController()))

        headerToolbarBack.setOnClickListener {onBackPressed()}
    }

    override fun displayBackButton(show: Boolean) {
        supportActionBar?.setDisplayHomeAsUpEnabled(show)
    }

    override fun hideAll() {
//        appBar.visibility=Toolbar.INVISIBLE
        Timber.e(" lay params : ${appBar.layoutParams.javaClass}")
        val params:CoordinatorLayout.LayoutParams = conductor_container.layoutParams as CoordinatorLayout.LayoutParams
        params.behavior = null
        conductor_container.requestLayout()
    }

    override val statusToolbar: ViewGroup
        get() = tbContainer.findViewById(R.id.status_toolbar_container)

    override val headerToolbar: ViewGroup
        get() = tbContainer.findViewById(R.id.switch_toolbar_container)

    override val headerToolbarBack
        get() = headerToolbar.findViewById<MaterialTextView>(R.id.back_button_chevron)


    override fun setToolbar(header:HeaderToolbarModel?,status:StatusToolbarModel?){
        appBar.setExpanded(false,false)
        if (header == null && status == null){
            hideAll()
            appBar.setGone(true)
            return
        } else appBar.setGone(false)
        showAll()
        appBarScrollable(false)
        headerToolbar.setGone(header==null)
        statusToolbar.setGone(status==null)
        header?.let {
            headerToolbarBack.setGone(!it.backButton)
            headerToolbarBack.text = it.backTitle?:""

            headerToolbar.toolbar_title.text = it.title?:""
            headerToolbar.include13.setGone(!(it.switcher?:false))
        }
        status?.let {
            statusToolbar.status_icon.setGone(it.state == null)
            statusToolbar.status_title.setGone(it.state == null)
            it.state?.let { state ->
                statusToolbar.status_icon.isEnabled = when(state){
                    StatusState.AVAILABLE -> true
                    else -> false
                }

                statusToolbar.status_title.text = when(state){
                    StatusState.AVAILABLE -> resources.getString(R.string.estate_avalibale)
                    StatusState.SOLD_OUT -> resources.getString(R.string.estate_sold_out)
                }
            }
            statusToolbar.mean_title.setGone(it.meanTitle == null)
            it.meanTitle?.let { title -> statusToolbar.mean_title.text = title}
        }

        val prms = appBar.layoutParams as CoordinatorLayout.LayoutParams
        prms.height = (if (!headerToolbar.isGone) headerToolbar.height else 0) +
                (if (!statusToolbar.isGone) statusToolbar.height else 0)
        appBar.layoutParams = prms
        appBar.requestLayout()
    }


    override fun hide() {
        bottomNavigation.setGone(true)
        bottomNavigation.setGone(true)
        val params = conductorContainer.layoutParams as CoordinatorLayout.LayoutParams
        params.bottomMargin = 0
        conductorContainer.requestLayout()
    }

    override fun show() {
        bottomNavigation.setGone(false)
        bottomNavigation.setGone(false)
        val params = conductorContainer.layoutParams as CoordinatorLayout.LayoutParams
        params.bottomMargin = resources.getDimensionPixelSize(R.dimen.bottom_navigation_height)
        conductorContainer.requestLayout()
    }



    override fun showAll() {
        topAppBar.visibility = View.VISIBLE
        val params = conductorContainer.layoutParams as CoordinatorLayout.LayoutParams
        params.behavior = AppBarLayout.ScrollingViewBehavior()
        conductorContainer.requestLayout()
    }

    override fun appBarScrollable(allow: Boolean) {
        val params = appBar.layoutParams as CoordinatorLayout.LayoutParams
        val behavior = params.behavior as DragControlAppBarLayoutBehaviour
        behavior.allowDrag=allow
    }

    override fun onBackPressed() {
        if (router.handleBack()) {
            Timber.d("In stack : " + router.backstackSize)
        } else {
            super.onBackPressed()
        }
    }
}

