package com.biganto.visual.roompark.conductor

import android.content.Context
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.inputmethod.InputMethodManager
import android.widget.Toast
import androidx.annotation.LayoutRes
import androidx.annotation.StringRes
import butterknife.ButterKnife
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.IBottomNavigation
import com.biganto.visual.roompark.base.ICollapsingToolBar
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.mosby.mvi.BigantoMviController
import com.biganto.visual.roompark.di.dagger.ActivityModule
import com.biganto.visual.roompark.util.monades.ExceptionString
import com.biganto.visual.roompark.util.view_utils.snackbar.ISnackBarProvider
import com.hannesdorfmann.mosby3.mvi.MviBasePresenter
import io.reactivex.disposables.CompositeDisposable
import timber.log.Timber

/**
 * Created by Vladislav Bogdashkin on 28.05.2018.
 */

abstract class BigantoBaseController<VS : BigantoBaseViewState,V: BigantoBaseContract<VS>,P : MviBasePresenter<V,VS>>
    : BigantoMviController<V, P> {

    constructor():super()

    constructor(args: Bundle):super(args)

    val inject by lazy { injectDependencies() }

    protected abstract fun injectDependencies()

    protected val detachDisposable = CompositeDisposable()

    abstract var injectedPresenter:P

    override fun createPresenter(): P = injectedPresenter

    @LayoutRes
    protected abstract fun getLayoutId(): Int

    lateinit var toolBar: ICollapsingToolBar
    lateinit var bottomNavigationController: IBottomNavigation
    lateinit var snackbar: ISnackBarProvider

    override fun onAttach(view: View) {
        Timber.d("On Attach $view")

        super.onAttach(view)
    }

    override fun onDetach(view: View) {
        Timber.d("On Detach $view" )
        detachDisposable.clear()
        super.onDetach(view)
    }

    private var isInjected = false

    override fun onContextAvailable(context: Context) {
        super.onContextAvailable(context)
        inject
    }

    abstract fun onViewBound(v:View)

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup): View {
        val view = inflater.inflate(getLayoutId(), container, false)
//        instantiate the view
        retainViewMode = RetainViewMode.RELEASE_DETACH
        ButterKnife.bind(this, view)
        (activity as RoomParkMainActivity).let{
            snackbar = ActivityModule.provideSnackBar(it)
            toolBar = it
            bottomNavigationController = it
            snackbar.bindRootView(toolBar.coordinatorLayout)
        }
        onViewBound(view)
        return view
    }

    protected fun View.hideKeyboard() {
        val inputMethodManager = applicationContext
            ?.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        inputMethodManager.hideSoftInputFromWindow(this.windowToken, 0)
    }

    protected fun View.showKeyboard() {
        val inputMethodManager = applicationContext?.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
        inputMethodManager.showSoftInput(this, 0)
    }

    protected fun showMessage(@StringRes msgResId: Int) {
        Toast.makeText(this.applicationContext, msgResId, Toast.LENGTH_SHORT).show()
    }

    override fun handleBack(): Boolean {
        router.popController(this)
        return true
//        return super.handleBack()
    }

    fun showError(e: ExceptionString) =
        e.selectHandler(snackbar::showSnackBar,snackbar::showSnackBar)

    protected val isTablet
        get() = resources?.getBoolean(R.bool.isTablet)?:false

    protected val catalogSpansCount
        get() = resources?.getInteger(R.integer.catalogSpans)?:1

    protected val toursSpansCount
        get() = resources?.getInteger(R.integer.catalogSpans)?:1
}