package com.biganto.visual.roompark.conductor.dialogs

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.LinearLayout
import android.widget.TextView
import androidx.annotation.LayoutRes
import com.biganto.visual.roompark.R
import com.bluelinelabs.conductor.Controller

/**
 * Created by Vladislav Bogdashkin on 09.04.2019.
 */

internal const val ALERT_DIALOG_MESSAGE_KEY = "BIGANTO_ALERT_MESSAGE"
internal const val ALERT_DIALOG_BUTTON_TEXT_KEY = "BIGANTO_ALERT_OK_BUTTON_TEXT"
internal const val ALERT_DIALOG_BUTTON_CANCEL_TEXT_KEY = "BIGANTO_ALERT_CANCEL_BUTTON_TEXT"

fun formBundle(message:String): Bundle {
    val b = Bundle()
    b.putString(ALERT_DIALOG_MESSAGE_KEY,message)
    return b
}

fun formBundle(message:String,buttonText:String): Bundle {
    val b = Bundle()
    b.putString(ALERT_DIALOG_MESSAGE_KEY,message)
    b.putString(ALERT_DIALOG_BUTTON_TEXT_KEY,buttonText)
    return b
}

fun formBundle(message:String,buttonText:String,cancelButtonText:String): Bundle {
    val b = Bundle()
    b.putString(ALERT_DIALOG_MESSAGE_KEY,message)
    b.putString(ALERT_DIALOG_BUTTON_TEXT_KEY,buttonText)
    b.putString(ALERT_DIALOG_BUTTON_CANCEL_TEXT_KEY,cancelButtonText)
    return b
}



open class AlertDialogController : Controller {

    constructor():super()
    constructor(args: Bundle):super(args)

    public constructor(alertMessage:String):super(formBundle(alertMessage))

    public constructor(alertMessage:String,buttonText: String):super(formBundle(alertMessage, buttonText))

    public constructor(alertMessage:String,buttonText: String,cancelText: String)
            :super(formBundle(alertMessage, buttonText,cancelText))

    public open var dismissActionDelegate = {router.popController(this);Unit}

    @LayoutRes
    fun getLayoutId() = R.layout.alert_dialog_screen_view

    private lateinit var messageTextView : TextView

    private lateinit var okButton : Button

    private lateinit var dismissButton : Button


    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup): View {
        val view = inflater.inflate(getLayoutId(), container, false)

        messageTextView=view.findViewById(R.id.alert_message_text_view)
        val alertMessage = args.getString(ALERT_DIALOG_MESSAGE_KEY)
        messageTextView.text=alertMessage

        okButton=view.findViewById(R.id.alert_ok_button)
        if (args.containsKey(ALERT_DIALOG_BUTTON_TEXT_KEY))
            okButton.text=args.getString(ALERT_DIALOG_BUTTON_TEXT_KEY)

        dismissButton=view.findViewById(R.id.alert_dismiss_button)
        val bContainer=view.findViewById<LinearLayout>(R.id.buttons_container)
        if (args.containsKey(ALERT_DIALOG_BUTTON_CANCEL_TEXT_KEY)){
            bContainer.weightSum=2f
            dismissButton.visibility=View.VISIBLE
            dismissButton.text=args.getString(ALERT_DIALOG_BUTTON_CANCEL_TEXT_KEY)
            dismissButton.setOnClickListener { router.popController(this) }
        }else{
            bContainer.weightSum=1f
            dismissButton.visibility=View.GONE
        }

        okButton.setOnClickListener {
            router.popController(this)
            dismissActionDelegate.invoke()
        }
        return view
    }
}