package com.biganto.visual.roompark.conductor.dialogs

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.annotation.LayoutRes
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.conductor.dialogs.change_handler.DialogChangeHandler
import com.biganto.visual.roompark.domain.model.PhotoResolutionModel
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonRecyclerAdapter
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonViewHolder
import com.bluelinelabs.conductor.Controller
import com.bluelinelabs.conductor.RouterTransaction
import com.google.android.material.textview.MaterialTextView
import io.reactivex.disposables.CompositeDisposable
import timber.log.Timber


/**
 * Created by Vladislav Bogdashkin on 09.04.2019.
 */

internal const val  PHOTOS_KEY = "CHHOSE_PHOTO_LIST_KEY"

private fun formBundle(photos: ArrayList<PhotoResolutionModel>): Bundle {
    val b = Bundle()
    b.putParcelableArrayList(PHOTOS_KEY,photos)
    return b
}

class ChooseResolutionDialogController : Controller {

    constructor(args: Bundle) : super(args)

    constructor(photos: ArrayList<PhotoResolutionModel>) : super(formBundle(photos))

    lateinit var recyclerView : RecyclerView

    private val detachDisposable = CompositeDisposable()

    override fun onDetach(view: View) {
        detachDisposable.clear()
        super.onDetach(view)
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup): View {
        val view = inflater.inflate(getLayoutId(), container, false)

        recyclerView = view.findViewById(R.id.photoSizesRecyclerView)
        recyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        recyclerView.adapter = PhotoSizeAdapter()
        recyclerView.itemAnimator = null
        recyclerView.addItemDecoration(
            DividerItemDecoration(activity,DividerItemDecoration.VERTICAL)
        )

        args.getParcelableArrayList<PhotoResolutionModel>(PHOTOS_KEY)?.let {
            (recyclerView.adapter as PhotoSizeAdapter).setItems(it)
        }

        detachDisposable.add(
        (recyclerView.adapter as PhotoSizeAdapter).onItemClicked.subscribe {
            Timber.d("gonna shit : $it")
            router.replaceTopController(RouterTransaction.with(
                PhotoDialogController(
                    it.url
                )
            )
                .popChangeHandler(DialogChangeHandler())
                .pushChangeHandler(DialogChangeHandler())
            )
        })

        view.findViewById<View>(R.id.cancel_button).setOnClickListener { handleBack() }

        return view
    }

    @LayoutRes
    fun getLayoutId() = R.layout.choose_size_modal_screen

    override fun handleBack(): Boolean {
        return router.popCurrentController()
    }


}

internal class PhotoSizeAdapter:CommonRecyclerAdapter<PhotoSizeViewHolder,PhotoResolutionModel>(){
    override val vhKlazz = PhotoSizeViewHolder::class
    override fun getVhLayout(): Int = R.layout.choose_size_viewholder
}

internal class PhotoSizeViewHolder(itemView: View) :CommonViewHolder<PhotoResolutionModel>(itemView){

    @BindView(R.id.size_text)
    lateinit var textView : MaterialTextView

    override fun onViewBound(model: PhotoResolutionModel) {
        Timber.d("resname: ${model.resName}")
        textView.text = "${model.resWidth}x${model.resHeight}"
    }
}
