package com.biganto.visual.roompark.data.local

import android.app.Application
import com.afollestad.rxkprefs.Pref
import com.afollestad.rxkprefs.rxkPrefs
import com.biganto.visual.androidplayer.data.repository.local.ILocalStore
import com.jakewharton.rxrelay2.BehaviorRelay
import dagger.Module
import dagger.Provides
import io.reactivex.Completable
import io.reactivex.Observable
import io.reactivex.disposables.CompositeDisposable
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Singleton

/**
 * Created by Vladislav Bogdashkin on 14.06.2018.
 */

@Module
class LocalStorage(){

//    @Binds
//    @Singleton
//    abstract fun provideStore(store:UserHolder):ILocalStore


    @Provides
    @Singleton
    fun provideStore(app : Application):ILocalStore = UserHolder(app)
}

class UserHolder @Inject constructor(val context : Application) : ILocalStore
{
    private val prefs = rxkPrefs(context)

    private val userSession : Pref<String> = prefs.string(RECENT_UUID_KEY, EMPTY_PREF_VALUE_KEY)

    private val sessionSubject = BehaviorRelay.create<UserState>()

    private val disposable = CompositeDisposable()
    init {
        Timber.d(" inited")
        disposable.add(userSession.observe().subscribe {
            sessionSubject.accept(
                if (it == EMPTY_PREF_VALUE_KEY) UserState.NotAuthenticated()
                else UserState.Authenticated(it)
            )
        })
        val s = userSession.get()
        sessionSubject.accept(
            if ( s== EMPTY_PREF_VALUE_KEY) UserState.NotAuthenticated()
            else UserState.Authenticated(s)

        )
    }


    companion object {
        const val RECENT_UUID_KEY = "com.biganto.visual.androidplayer.LAST_USER_UUD"
        const val EMPTY_PREF_VALUE_KEY = "NO_ACTIVE_SESSION"

        const val SCROLL_LIST_TO_TOUR_KEY="SCROLL_LIST_TO_TOUR_ID"


    }

    override fun setDownloadListDestinationTourId(tourId:String) =
            prefs.string(SCROLL_LIST_TO_TOUR_KEY,tourId)
                .observe()

    override fun getDownloadListDestinationTourId() =
            prefs.string(SCROLL_LIST_TO_TOUR_KEY,"")
                .observe()

    override fun recentUserObs() = userSession.observe()

    override fun recentUser(): Observable<UserState> = sessionSubject

    override fun setRecentUser(uuid: String?) =
        Completable.defer {
            Completable.fromCallable {
                userSession.set(uuid ?: EMPTY_PREF_VALUE_KEY) }
                .doOnComplete { Timber.d("complete save null value") }
        }
}

sealed class UserState{
    class NotAuthenticated:UserState()
    data class Authenticated(val uuid: String):UserState()
}

data class UserPrefModel(
        val uuid:String
        ,val name:String
        ,val email:String
        ,val timeZone:String)
