package com.biganto.visual.roompark.data.repository.api.retrofit.util

import com.biganto.visual.roompark.data.repository.api.retrofit.raw.ErrorRaw
import com.biganto.visual.roompark.domain.custom_exception.parseException
import com.google.gson.*
import timber.log.Timber
import java.lang.reflect.Type

/**
 * Created by Vladislav Bogdashkin on 12.11.2019.
 */


class CustomExceptionDeserializer<T> : JsonDeserializer<T> {
    override fun deserialize(json: JsonElement, typeOfT: Type, ctx: JsonDeserializationContext): T {
        val vals = ArrayList<ErrorRaw>()


        //region valid case
        when(json){
            is JsonObject -> if (!json.asJsonObject.keySet().contains("errors"))
                return Gson().newBuilder().create().fromJson(json,typeOfT)
            is JsonArray -> return Gson().newBuilder().create().fromJson(json,typeOfT)
        }
        //endregion valid case

        //region parse errorList
        val errorList = json.asJsonObject.get("errors")

        Timber.w("api errorlist: $errorList")

        when(json) {
            is  JsonArray -> errorList.asJsonArray.forEach{
                vals.add(ctx.deserialize<T>(it, ErrorRaw::class.java) as ErrorRaw)
            }
            is JsonObject -> json.asJsonObject.get("errors").asJsonArray.forEach {
                vals.add(ctx.deserialize<T>(it, ErrorRaw::class.java) as ErrorRaw)
            }
            else -> throw RuntimeException("Unexpected JSON type: " + json.javaClass)
        }
        if (vals.size==0)
            error("unhandled api result : $json")

        throw parseException(vals.first()) //-> for now throws only first!
        //endregion
    }
}