package com.biganto.visual.roompark.data.repository.db.requrey.utils;

import com.biganto.visual.roompark.data.repository.db.requrey.TitledPhoto;

import java.util.ArrayList;
import java.util.List;

import io.requery.Converter;
import timber.log.Timber;

/**
 * Created by Vladislav Bogdashkin on 04.07.2018.
 */


public class TitledPhotoListConverter implements Converter<List<TitledPhoto>, String> {

    private static final String stringDelimeter="♀°♀";

    @SuppressWarnings("unchecked")
    @Override
    public Class<List<TitledPhoto>> getMappedType() {
        return (Class) List.class;
    }

    @Override
    public Class<String> getPersistedType() {
        return String.class;
    }

    @Override
    public Integer getPersistedSize() {
        return null;
    }

    @Override
    public String convertToPersisted(List<TitledPhoto> value) {
        if (value == null) {
            return "";
        }
        StringBuilder sb = new StringBuilder();
        int index = 0;
        for (TitledPhoto str: value) {
            if (index > 0) {
                sb.append(stringDelimeter);
            }
            sb.append(str.delimiterString());
            index++;
        }
        return sb.toString();
    }

    @Override
    public List<TitledPhoto> convertToMapped(Class<? extends List<TitledPhoto>> type,
                                String value) {
        try {
            ArrayList<TitledPhoto> list = new ArrayList<>();
            if (value != null && !value.isEmpty())
                for (String s : value.split(stringDelimeter))
                    list.add(new TitledPhoto(s));
            return list;
        }
        catch (Exception e){
            Timber.e("Wrong stored data format! {%s}",value);
            return new ArrayList<TitledPhoto>(0);
        }

    }
}