package com.biganto.visual.roompark.data.service.user_session

import com.biganto.visual.androidplayer.data.repository.local.ILocalStore
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.data.local.UserState
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.domain.custom_exception.CustomApiException
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 19.11.2019.
 */


//@Singleton
class  UserSessionService @Inject constructor(
    private val local:ILocalStore,
    private val db:IDb
):IUserSession{

    private var _token:String? = null

    override val token = _token?: ""


    val disposable =
        local.recentUserObs()
            .flatMap { db.fetchUser(it.toInt()) }
            .subscribe { _token = it.authToken }

    val disposable2 =
        local.recentUserObs()
            .doOnNext { Timber.w("got new user $it") }
            .flatMap { db.userObservableResult(it.toInt())
                .doOnNext { Timber.w("obs db user $it") }
                .flatMap{ reactiveResult -> reactiveResult.observable()
                    .doOnNext { Timber.w("flatmapUser db user $it") } }
            }
            .doOnNext { Timber.w("subscribe user $it") }
            .subscribe { _token = it.authToken }





    fun currentUser() =
        local.recentUser().map {
            when(it){
                is UserState.Authenticated ->it.uuid
                is UserState.NotAuthenticated ->
                    CustomApiException
                        .UnknownCustomApiException(-1, R.string.unauthorized_user_request,null)
                else -> CustomApiException
                    .UnknownCustomApiException(-1, R.string.unauthorized_user_request,null)
            }
        }

    fun getUser(uuid:Int) = db.fetchUser(uuid).map { it.authToken }

}

class UnauthorizedUser(override var message:String,val userId:String): Exception(message)

interface IUserSession{
    val token :String

}

sealed class UserSessionState(){
    class Active(uuid:String):UserSessionState()
    class Inactive():UserSessionState()
}