package com.biganto.visual.roompark.data.service.version_control

import android.content.Context
import android.os.Parcel
import android.os.Parcelable
import com.biganto.visual.roompark.BuildConfig
import com.biganto.visual.roompark.data.repository.api.retrofit.raw.AppVersionRaw
import com.jakewharton.rxrelay2.BehaviorRelay
import dagger.Module
import dagger.Provides
import io.reactivex.Observable
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Singleton


/**
 * Created by Vladislav Bogdashkin on 12.10.2018.
 */

@Module
class AppVersionModule @Inject constructor(val context: Context){

    private val manager = AppVersionManager(context)

    @Provides
    @Singleton
    fun provideVersionListener(): IAppVersionControl {
        return manager
    }

    @Provides
    @Singleton
    fun provideAppVersionNotifier(): IAppVersionNotifier {
        return manager
    }

}

@Singleton
class AppVersionManager @Inject constructor(val context: Context)
    : IAppVersionControl, IAppVersionNotifier {

    override fun processAppVersionResponse(): Observable<UpdateAppModel> {
        return notifier
    }

//    val api : IApi by lazy {
//        RoomParkApplication.component.api()
//    }

//    lateinit var notifier : IAppVersionNotifier


    private val notifier = BehaviorRelay.create<UpdateAppModel>()

    private var appVersionStatus = AppVersionStatus.UNCHECKED

    override val appVersionValid: Boolean
        get() = appVersionStatus == AppVersionStatus.VALID

    private var buildVersion: String = BuildConfig.VERSION_NAME

    override fun validateVersion() {
//        api.getAppVersion().subscribe(::mergeVersion, Timber::e).dispose()
    }

    private fun mergeVersion(apiAppVersion: AppVersionRaw){
        Timber.d("Got new version info (current: $buildVersion, incoming: $apiAppVersion")
        appVersionStatus = AppVersionStatus.VALID

        // region Code in spoiler not used in current version
//        if (apiAppVersion.current_version!=buildVersion)
//            appVersionStatus = AppVersionStatus.OUTDATED
//        if (apiAppVersion.critical && appVersionStatus == AppVersionStatus.OUTDATED)
//            appVersionStatus = AppVersionStatus.CRITICAL
        //endregion

        if (apiAppVersion.critical && appVersionStatus == AppVersionStatus.OUTDATED)
            appVersionStatus = AppVersionStatus.CRITICAL

        if (appVersionStatus == AppVersionStatus.CRITICAL)
            sendIntentToUpdate(apiAppVersion)
    }

    private fun sendIntentToUpdate(apiAppVersion: AppVersionRaw){
        val updInfo = UpdateAppModel(
                currentVersion = buildVersion,
                newVersion = apiAppVersion.current_version,
                isCritical = apiAppVersion.critical,
                downloadUrl = apiAppVersion.download_url,
                message = apiAppVersion.message
        )
        notifier.accept(updInfo)
    }

}

interface IAppVersionControl{
    val appVersionValid: Boolean
    fun validateVersion()
}

interface IAppVersionNotifier{
    fun processAppVersionResponse():Observable<UpdateAppModel>
}

enum class AppVersionStatus{
    OUTDATED,
    CRITICAL,
    VALID,
    UNCHECKED
}

data class UpdateAppModel(
        val currentVersion : String,
        val newVersion : String,
        val isCritical : Boolean,
        val message : String?,
        val downloadUrl : String?
) : Parcelable {
    constructor(parcel: Parcel) : this(
            parcel.readString(),
            parcel.readString(),
            parcel.readByte() != 0.toByte(),
            parcel.readString(),
            parcel.readString()) {
    }

    override fun writeToParcel(parcel: Parcel, flags: Int) {
        parcel.writeString(currentVersion)
        parcel.writeString(newVersion)
        parcel.writeByte(if (isCritical) 1 else 0)
        parcel.writeString(message)
        parcel.writeString(downloadUrl)
    }

    override fun describeContents(): Int {
        return 0
    }

    companion object CREATOR : Parcelable.Creator<UpdateAppModel> {
        override fun createFromParcel(parcel: Parcel): UpdateAppModel {
            return UpdateAppModel(parcel)
        }

        override fun newArray(size: Int): Array<UpdateAppModel?> {
            return arrayOfNulls(size)
        }
    }
}

class AppVersionNotCheckedException(message:String): Exception(message)