package com.biganto.visual.roompark.domain.interactor

import com.biganto.visual.roompark.domain.model.AlbumSortedModel
import com.biganto.visual.roompark.domain.use_case.AlbumsUseCase
import io.reactivex.Observable
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 09.10.2019.
 */

class PhotoInteractor @Inject constructor(
    private val useCase: AlbumsUseCase) {

    fun fetchHeaderAlbums() = useCase.getProgressAlbums()
    fun fetchAlbumPhotos(parentId: Int): Observable<List<AlbumSortedModel>> =
        useCase.getChildAlbums(parentId)
            .flatMap {
                Observable.fromIterable(it).flatMap { alb ->
                    useCase.getPhotos(alb.albumId).take(1)
                        .map { photos ->
                            AlbumSortedModel(
                                alb.title,
                                alb.published,
                                alb.albumId,
                                photos
                            )
                        }
                }.toList().toObservable()
            }
            .debounce(80L, TimeUnit.MILLISECONDS) // to reduce double list draw effect

    fun fetchPhotos(albumId: Int) =
        useCase.getPhotos(albumId).take(1)

    fun fetchPhoto(photoId:Int) = useCase.getPhotoByid(photoId)

    fun getAlbum(albumId:Int) = useCase.getDirectAlbum(albumId)
}



