package com.biganto.visual.roompark.domain.model

import android.os.Parcel
import android.os.Parcelable
import com.biganto.visual.roompark.data.repository.db.requrey.model.GalleryPhotoEntity
import com.biganto.visual.roompark.data.repository.db.requrey.model.ImageAlbumEntity
import timber.log.Timber
import java.util.*

/**
 * Created by Vladislav Bogdashkin on 23.09.2019.
 */


data class AlbumPreviewModel(
    val albumId:Int,
    val parentId:Int,
    val title:String,
    val published: Date,
    val previewUrl:String,
    val isRead:Boolean
)

data class AlbumPhotoPreviewModel(
    val albumId:Int,
    val parentId:Int,
    val published: Date,
    val title:String,
    val previewUrl:String,
    val isRead:Boolean
)

data class AlbumSortedModel(
    val title:String,
    val published: Date,
    val albumId: Int,
    val items:List<PhotoModel>
)


data class PhotoListModel(val items:List<PhotoModel>)

data class PhotoModel(
    val photoId:Int,
    val albumId:Int,
    val description:String?,
    val sort:Int,
    val resolutionList:List<PhotoResolutionModel>
){
    fun optimalResolution(width:Int, height:Int) =
        resolutionList
            .asSequence()
            .filter { it.resWidth >= width || it.resWidth >= height }
            .filter { it.resHeight >= height || it.resHeight >= width }
            .onEach { Timber.d("res filtered: ${it.resWidth} / ${it.resHeight}") }
            .minBy { it.resHeight * it.resWidth }
            ?: resolutionList.maxBy { it.resHeight * it.resWidth }!!
}

data class PhotoResolutionModel(
    val resName:String,
    val url:String,
    val resWidth:Int,
    val resHeight:Int
) : Parcelable {
    constructor(parcel: Parcel) : this(
        parcel.readString(),
        parcel.readString(),
        parcel.readInt(),
        parcel.readInt()
    )

    override fun writeToParcel(parcel: Parcel, flags: Int) {
        parcel.writeString(resName)
        parcel.writeString(url)
        parcel.writeInt(resWidth)
        parcel.writeInt(resHeight)
    }

    override fun describeContents(): Int {
        return 0
    }

    companion object CREATOR : Parcelable.Creator<PhotoResolutionModel> {
        override fun createFromParcel(parcel: Parcel): PhotoResolutionModel {
            return PhotoResolutionModel(parcel)
        }

        override fun newArray(size: Int): Array<PhotoResolutionModel?> {
            return arrayOfNulls(size)
        }
    }
}


fun fromEntity(entity: ImageAlbumEntity):AlbumPreviewModel =
    AlbumPreviewModel(
        albumId = entity.id,
        parentId = -1,
        title = entity.title,
        published = entity.published,
        previewUrl = entity.preview,
        isRead = false
    )

fun fromEntity(entity: GalleryPhotoEntity) =
    PhotoModel(
        photoId = entity.id,
        albumId = entity.album_id,
        description = entity.description,
        sort = entity.sort,
        resolutionList = List(entity.resolutions.size){
            PhotoResolutionModel(
            resName = entity.resolutions[it].res_name,
                url = entity.resolutions[it].url,
                resWidth = entity.resolutions[it].width,
                resHeight = entity.resolutions[it].height
        )}
    )