package com.biganto.visual.roompark.presentation.screen.albums.util

import android.view.View
import androidx.recyclerview.widget.RecyclerView
import androidx.recyclerview.widget.StaggeredGridLayoutManager
import androidx.recyclerview.widget.StaggeredGridLayoutManager.GAP_HANDLING_MOVE_ITEMS_BETWEEN_SPANS
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.domain.model.AlbumSortedModel
import com.biganto.visual.roompark.domain.model.PhotoModel
import com.biganto.visual.roompark.domain.model.PhotoResolutionModel
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonRecyclerAdapter
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonViewHolder
import com.biganto.visual.roompark.util.view_utils.image_view.RoundedImageView
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.google.android.material.textview.MaterialTextView
import com.jakewharton.rxbinding3.view.detaches
import com.jakewharton.rxrelay2.BehaviorRelay
import io.reactivex.disposables.CompositeDisposable
import timber.log.Timber

/**
 * Created by Vladislav Bogdashkin on 16.10.2019.
 */


class AlbumListAdapter : CommonRecyclerAdapter<AlbumViewHolder, AlbumSortedModel>() {
    override val vhKlazz = AlbumViewHolder::class

    override fun getVhLayout(): Int = R.layout.date_album_viewholder

    private val disp = CompositeDisposable()

    val photoObs = BehaviorRelay.create<PhotoModel>()

    override fun onBindViewHolder(holder: AlbumViewHolder, position: Int) {
        super.onBindViewHolder(holder, position)
        holder.setIsRecyclable(false)
        disp.addAll(
            holder.photoClicksObservable()
                ?.doOnNext { Timber.d("photo clicked $it") }
                ?.takeUntil( holder.itemView.detaches() )
                ?.subscribe(photoObs)
        )
    }
}

    class AlbumViewHolder(itemView: View) : CommonViewHolder<AlbumSortedModel>(itemView) {

        @BindView(R.id.date_title_textview)
        lateinit var albumTitle: MaterialTextView
        @BindView(R.id.photos_recyclerview)
        lateinit var photosRecyclerView: RecyclerView
//        @BindView(R.id.camStatusIcon) lateinit var camStatusIcon:ImageView

        private var adapter:PhotosAdapter? = null

        fun photoClicksObservable() =
            adapter?.onItemClicked

        override fun onViewBound(model: AlbumSortedModel) {
            albumTitle.text = model.title
            photosRecyclerView.isNestedScrollingEnabled = false
            photosRecyclerView.layoutManager =
                StaggeredGridLayoutManager(4, RecyclerView.VERTICAL)

            (photosRecyclerView.layoutManager as StaggeredGridLayoutManager)
                .gapStrategy = GAP_HANDLING_MOVE_ITEMS_BETWEEN_SPANS
            adapter = PhotosAdapter()
            photosRecyclerView.adapter = adapter
            photosRecyclerView.itemAnimator = null
            adapter?.setItems(model.items)
        }
    }

class PhotosAdapter : CommonRecyclerAdapter<PhotosViewHolder, PhotoModel>() {
    override val vhKlazz = PhotosViewHolder::class

    override fun getVhLayout(): Int = R.layout.photo_preview_viewholder
}

class PhotosViewHolder(itemView: View) : CommonViewHolder<PhotoModel>(itemView) {

    @BindView(R.id.photo_preview_imageview) lateinit var photoPreview: RoundedImageView

    override fun onViewBound(model: PhotoModel) {
        model.resolutionList.lowlest()?.let {
            Glide.with(itemView)
                .asBitmap()
                .load(it.url)
                .centerCrop()
                .fitCenter()
                .diskCacheStrategy(DiskCacheStrategy.ALL)
                .into(photoPreview)
        }}
}

fun List<PhotoResolutionModel>.lowlest() =
    this.minBy { it.resWidth * it.resHeight }