package com.biganto.visual.roompark.presentation.screen.albums.util

import android.content.Context
import android.graphics.SurfaceTexture
import android.util.AttributeSet
import android.view.TextureView
import androidx.recyclerview.widget.RecyclerView
import timber.log.Timber

/**
 * Created by Vladislav Bogdashkin on 22.12.2019.
 */

class BubbleSlider @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : TextureView(context, attrs, defStyleAttr)
    ,TextureView.SurfaceTextureListener{

    private var texThread: RendererThread?=null

    private var recyclerView:RecyclerView? = null

    private var actualPosition = -1

    private fun invalidatePosition(){
        val child = recyclerView?.findViewHolderForAdapterPosition(actualPosition)
        if (child == null) texThread?.setCenter(-9999)
        else texThread?.setCenter(child.itemView.x.toInt()+child.itemView.width/ 2)
    }

    fun setUpView(view:RecyclerView) {
        recyclerView = view
        recyclerView?.setOnScrollChangeListener { v, scrollX, scrollY, oldScrollX, oldScrollY ->
            invalidatePosition()
        }
    }

    fun onCurrentItemChanged(newPosition:Int){
        actualPosition = newPosition
        invalidatePosition()
        texThread?.setIndex(actualPosition)
    }


    fun processSliding(_position:Float){
        texThread?.setSlidePosition(_position)
    }

    init {
        surfaceTextureListener=this
    }


    override fun onSizeChanged(w: Int, h: Int, oldw: Int, oldh: Int) {
        Timber.d("On Size Chagned")
        super.onSizeChanged(w, h, oldw, oldh)
    }

    override fun onSurfaceTextureSizeChanged(surface: SurfaceTexture?, width: Int, height: Int) {
        print(" onSurfaceTextureSizeChanged")
        texThread?.setSize(width,height)
        //Ignored
    }

    override fun onSurfaceTextureUpdated(surface: SurfaceTexture?) {
        print(" onSurfaceTextureUpdated")
        //Ignored
    }

    override fun onSurfaceTextureDestroyed(surface: SurfaceTexture?): Boolean {
        texThread?.isStopped=true
        return true
    }

    override fun onSurfaceTextureAvailable(surface: SurfaceTexture?, width: Int, height: Int) {
        Timber.d("avaliable surf: $surface")
        surface?.let {
            if (texThread?.isAlive==true)
                return
            texThread = RendererThread(context,it)
            texThread?.setSize(width,height)
            texThread?.start()
        }
    }

    override fun onDetachedFromWindow() {
        super.onDetachedFromWindow()
        texThread?.isStopped=true
    }
}


