package com.biganto.visual.roompark.presentation.screen.article

import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import androidx.core.os.bundleOf
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import butterknife.OnClick
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.conductor.dialogs.PhotoDialogController
import com.biganto.visual.roompark.conductor.dialogs.change_handler.DialogChangeHandler
import com.biganto.visual.roompark.presentation.screen.article.util.ArticleBottomPhotoAdapter
import com.biganto.visual.roompark.presentation.screen.article.util.HtmlPageAdapter
import com.biganto.visual.roompark.presentation.screen.article.util.HtmlTag
import com.biganto.visual.roompark.util.extensions.formatToSimple
import com.biganto.visual.roompark.util.extensions.setGone
import com.bluelinelabs.conductor.RouterTransaction
import com.bumptech.glide.Glide
import com.bumptech.glide.load.engine.DiskCacheStrategy
import com.google.android.material.textview.MaterialTextView
import jp.wasabeef.glide.transformations.BlurTransformation
import timber.log.Timber
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

private const val ARTICLE_ID = "DIRECT_ARTICLE_ID_KEY"

class ArticleScreenController :
    BigantoBaseController<ArticleScreenViewState
            , ArticleScreen
            , ArticleScreenPresenter>
    , ArticleScreen {

    constructor(args:Bundle):super(args)

    constructor(id: Int) : super(bundleOf(ARTICLE_ID to id))

    override fun injectDependencies() {
        getComponent()
    }


    @Inject
    override lateinit var injectedPresenter: ArticleScreenPresenter

    @BindView(R.id.articleTitle)
    lateinit var title:MaterialTextView

    @BindView(R.id.articleHeaderBlock)
    lateinit var headerBlock: ViewGroup

    @BindView(R.id.articleBodyRecyclerView)
    lateinit var articleRecyclerView: RecyclerView

    @BindView(R.id.article_photos_recylcerView)
    lateinit var bottomPhotosRecyclerView: RecyclerView

    private val blurPreview:ImageView by lazy {
       headerBlock.findViewById<ImageView>(R.id.articlePreviewBlurred)
    }

    private val articlePreview:ImageView by lazy {
        headerBlock.findViewById<ImageView>(R.id.articlePreview)
    }

    val articleDate:MaterialTextView by lazy {
        headerBlock.findViewById<MaterialTextView>(R.id.articleDate)
    }

    @OnClick(R.id.articleCloseButton)
    fun onCloseArticle(){
        handleBack()
    }

    private fun bindRecycler() {
        articleRecyclerView.isNestedScrollingEnabled = true
        articleRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        articleRecyclerView.adapter = HtmlPageAdapter()
        articleRecyclerView.itemAnimator = null

        bottomPhotosRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.HORIZONTAL, false)
        bottomPhotosRecyclerView.adapter = ArticleBottomPhotoAdapter()
        bottomPhotosRecyclerView.itemAnimator = null

    }

    override fun onViewBound(v: View) {
        toolBar.setToolbar()
        bottomNavigationController.hide()
        bindRecycler()
        detachDisposable.add(
            (bottomPhotosRecyclerView.adapter as ArticleBottomPhotoAdapter)
                .onItemClicked
                .subscribe { router.pushController(
                    RouterTransaction.with(PhotoDialogController(it.url))
                        .pushChangeHandler(DialogChangeHandler())
                        .popChangeHandler(DialogChangeHandler())
                )
                }
        )
    }

    override fun render(viewState: ArticleScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is ArticleScreenViewState.Idle -> render(viewState)
            is ArticleScreenViewState.ArticleLoaded -> render(viewState)
        }
    }

    private fun render(viewState: ArticleScreenViewState.Idle){

    }

    private fun render(viewState: ArticleScreenViewState.ArticleLoaded) {

        title.text = viewState.item.title

        val z = viewState.item.htmlBody
            .replace("<\\br>","\n")
            .replace("<br>","\n")
            .replace("</br>","\n")
            .replace("<p>","")
            .replace("</p>","")

        val tags = z.lines().map {
            if (it.startsWith("<img"))
                HtmlTag.ImageSource(it.substringAfter("src=\"").substringBefore("\""))
            else HtmlTag.Text(it)
        }.toList()

        (articleRecyclerView.adapter as HtmlPageAdapter).setItems(tags)

        articleDate.text = viewState.item.published.formatToSimple

        bottomPhotosRecyclerView.setGone(viewState.item.bottomPhotoList.isNullOrEmpty())
        viewState.item.bottomPhotoList?.let {
            (bottomPhotosRecyclerView.adapter as ArticleBottomPhotoAdapter).setItems(it)
        }

        Glide.with(blurPreview)
            .load(viewState.item.previewUrl)
            .transform(BlurTransformation(40,4))
            .diskCacheStrategy(DiskCacheStrategy.ALL)
            .into(blurPreview)

        Glide.with(articlePreview)
            .load(viewState.item.previewUrl)
            .diskCacheStrategy(DiskCacheStrategy.ALL)
            .into(articlePreview)
    }

    private fun getComponent() = DaggerArticleScreenComponent.factory()
        .create(RoomParkApplication.component
            ,activity as RoomParkMainActivity
            ,args.getInt(ARTICLE_ID))
        .inject(this)

    override fun getLayoutId(): Int = R.layout.feed_read_screen


}

