package com.biganto.visual.roompark.presentation.screen.article

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.ArticleInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class ArticleScreenPresenter @Inject constructor(
    private val interactor: ArticleInteractor,
    @Named(ARTICLE_SCREEN_ARTICLEID) private val  selectedArticleId:Int
)
    : BigantoBasePresenter<ArticleScreen, ArticleScreenViewState>() {
    override fun defaultErrorViewStateHandler() =
        {e:ExceptionString -> ArticleScreenViewState.SomeError(e)}


    override fun bindIntents() {

        val prefetchCards = interactor.fetchArticle(selectedArticleId)
            .map { ArticleScreenViewState.ArticleLoaded(it) }

        val setRead = interactor.setArticleRead(selectedArticleId)
            .andThen(Observable.just(ArticleScreenViewState.Idle()))


        val state = restoreStateObservable
            .mergeWith(prefetchCards)
            .mergeWith(setRead)
            .doOnError{ Timber.e(it)}
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(ArticleScreenViewState::class.java), ArticleScreen::render)
    }

}