package com.biganto.visual.roompark.presentation.screen.estate

import android.net.Uri
import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import android.webkit.WebView
import android.widget.LinearLayout
import androidx.core.os.bundleOf
import androidx.core.view.isNotEmpty
import androidx.core.widget.NestedScrollView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.base.StatusState
import com.biganto.visual.roompark.base.StatusToolbarModel
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.domain.model.*
import com.biganto.visual.roompark.presentation.screen.estate.util.FlatInfoAdapter
import com.biganto.visual.roompark.util.extensions.setGone
import com.biganto.visual.roompark.util.extensions.startUrl
import com.biganto.visual.roompark.util.view_utils.image_view.RoundedImageView
import com.bumptech.glide.Glide
import com.google.android.material.switchmaterial.SwitchMaterial
import com.google.android.material.tabs.TabLayout
import com.google.android.material.textview.MaterialTextView
import com.jakewharton.rxbinding3.material.selections
import com.jakewharton.rxbinding3.view.clicks
import com.jakewharton.rxbinding3.view.scrollChangeEvents
import com.jakewharton.rxbinding3.widget.checkedChanges
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

const val SELECTED_ESTATE_ID_KEY = "SELECTED_ESTATE_INDEX"


class EstateScreenController  :
    BigantoBaseController<EstateScreenViewState
        , EstateScreen
        , EstateScreenPresenter>
    , EstateScreen {


    override fun showCommonInfo(): Observable<Int> =
        commonInfoTab.clicks()
            .doOnNext {
                commonInfoTab.background = resources?.getDrawable(R.drawable.bottom_line_text_view)
                explicationTab.background = null
            }
            .map { 1 }
            .observeOn(AndroidSchedulers.mainThread())

    override fun showExplication(): Observable<Int> =
        explicationTab.clicks()
            .doOnNext {
                explicationTab.background = resources?.getDrawable(R.drawable.bottom_line_text_view)
                commonInfoTab.background = null
            }
            .map { planTypesTabLayout.selectedTabPosition }
            .observeOn(AndroidSchedulers.mainThread())

    private fun ViewGroup.switchMatch() =
        this.findViewById<SwitchMaterial>(R.id.switch1)
            .checkedChanges()
            .filter {planTypesTabLayout.isNotEmpty()}
            .filter{planTypesTabLayout.selectedTabPosition>=0}
            .map {
                Pair(planTypesTabLayout.selectedTabPosition,it)
            }

    override fun switchSizes(): Observable<Pair<Int, Boolean>> =
        sizesSwitcher.switchMatch()

    override fun switchWalls(): Observable<Pair<Int, Boolean>> =
        wallsSwitcher.switchMatch()

    override fun switchFurniture(): Observable<Pair<Int, Boolean>> =
        furnSwitcher.switchMatch()

    override fun switchElectric(): Observable<Pair<Int, Boolean>> =
        electricSwitcher.switchMatch()


    override fun planTypesTabSelected(): Observable<Int> =
        planTypesTabLayout.selections()
            .debounce (200L, TimeUnit.MILLISECONDS)
            .map { it.position }
            .observeOn(AndroidSchedulers.mainThread())


    constructor(args: Bundle):super(args)

    constructor(id: Int) : super(bundleOf(SELECTED_ESTATE_ID_KEY to id))

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: EstateScreenPresenter

    @BindView(R.id.flatTypesCustomView)
    lateinit var flatTypeView: ViewGroup

    @BindView(R.id.planTypesTabs)
    lateinit var planTypesTabLayout: TabLayout

    @BindView(R.id.flat_plan_webview)
    lateinit var planWebView: WebView

    @BindView(R.id.flat_nested_scroll)
    lateinit var flatScroll: NestedScrollView

    @BindView(R.id.flat_content_recycler_view)
    lateinit var flatInfoRecyclerView: RecyclerView

    @BindView(R.id.flat_title)
    lateinit var flatTitle: MaterialTextView

    @BindView(R.id.sizes_switch_container)
    lateinit var sizesContainer: LinearLayout
    @BindView(R.id.sizes_switch_header)
    lateinit var sizesTitle: MaterialTextView
    @BindView(R.id.sizes_switcher)
    lateinit var sizesSwitcher: ViewGroup

    @BindView(R.id.electricity_switch_container)
    lateinit var electricContainer: LinearLayout
    @BindView(R.id.electricity_switch_header)
    lateinit var electricTitle: MaterialTextView
    @BindView(R.id.electricity_switcher)
    lateinit var electricSwitcher: ViewGroup

    @BindView(R.id.furniture_switch_container)
    lateinit var furnContainer: LinearLayout
    @BindView(R.id.furniture_switch_header)
    lateinit var furnTitle: MaterialTextView
    @BindView(R.id.furniture_switcher)
    lateinit var furnSwitcher: ViewGroup

    @BindView(R.id.walls_switch_container)
    lateinit var wallsContainer: LinearLayout
    @BindView(R.id.walls_switch_header)
    lateinit var wallsTitle: MaterialTextView
    @BindView(R.id.walls_switcher)
    lateinit var wallsSwitcher: ViewGroup


    @BindView(R.id.common_info_tab)
    lateinit var commonInfoTab: MaterialTextView

    @BindView(R.id.explication_tab)
    lateinit var explicationTab: MaterialTextView

    @BindView(R.id.site_link_viewholder)
    lateinit var siteLink: View

    @BindView(R.id.start_tour_card)
    lateinit var startTour: View

    @BindView(R.id.info_tab_divider)
    lateinit var infoTabDivicder: View

    @BindView(R.id.start_tour_card_divider)
    lateinit var startTourDivider: View

    @BindView(R.id.sizes_divider)
    lateinit var sizesDivider: View
    @BindView(R.id.electricity_divider)
    lateinit var electricityDivider: View
    @BindView(R.id.walls_divider)
    lateinit var wallsDivider: View
    @BindView(R.id.furniture_divider)
    lateinit var furnitureDivider: View


    @BindView(R.id.start_tour_image_view) lateinit var tourScreen: RoundedImageView

    private fun setToolbar(){
        toolBar.setToolbar(null, StatusToolbarModel())
        toolBar.statusToolbar.findViewById<View>(R.id.back_cross).let {
            detachDisposable.add(
                it.clicks()
                    .subscribeOn(AndroidSchedulers.mainThread())
                    .subscribe { handleBack() }
            )
        }
        detachDisposable.add(
            flatScroll.scrollChangeEvents()
//                .debounce (25,TimeUnit.MILLISECONDS)
                .subscribeOn(AndroidSchedulers.mainThread())
                .subscribe {
                        if (it.scrollY > flatTitle.measuredHeight) {
                            val status = estateModel?.to(
                                StatusToolbarModel(
                                    StatusState.AVAILABLE
                                    , null
                                    , resources?.getString(
                                        estateModel?.type?.typeShortString() ?: -1
                                        , estateModel?.number
                                    )
                                )
                            )
                            toolBar.setToolbar(
                                null, status = status?.second
                            )
                        } else toolBar.setToolbar(
                            null,
                            StatusToolbarModel(
                                StatusState.AVAILABLE, null, null
                            )
                        )
                }
        )
    }

    private fun bindRecycler() {
        flatInfoRecyclerView.isNestedScrollingEnabled = true
        flatInfoRecyclerView.layoutManager =
            LinearLayoutManager(activity, RecyclerView.VERTICAL, false)
        flatInfoRecyclerView.adapter = FlatInfoAdapter()
        flatInfoRecyclerView.itemAnimator = null

    }

    override fun onViewBound(v: View) {
        bottomNavigationController.hide()
        setToolbar()
        bindRecycler()
    }

    override fun render(viewState: EstateScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is EstateScreenViewState.Idle -> render(viewState)
            is EstateScreenViewState.LoadEstate -> render(viewState)
            is EstateScreenViewState.LoadPlanTypes -> render(viewState)
            is EstateScreenViewState.LoadPlan -> render(viewState)
            is EstateScreenViewState.PlanTypeSelected -> render(viewState)
            is EstateScreenViewState.SomeError -> render(viewState)
            is EstateScreenViewState.ShowEstateInfo -> render(viewState)
        }
    }

    private fun render(viewState: EstateScreenViewState.Idle){

    }

    private fun render(viewState: EstateScreenViewState.SomeError) =
        showError(viewState.exception)

    private var estateModel : EstateModel? = null

    private fun render(viewState: EstateScreenViewState.LoadEstate) {
        estateModel = viewState.estate
        toolBar.setToolbar(
            null, StatusToolbarModel(StatusState.AVAILABLE,null, null)
        )
        flatTitle.text =  resources?.getString(viewState.estate.type.typeShortString()
            ,viewState.estate.number)

        siteLink.setGone(viewState.estate.url == null)
        viewState.estate.url?.let {url ->
            siteLink.setOnClickListener{
                activity?.startUrl(url)
            }
        }
        startTour.setGone(viewState.estate.multitourId == null)
        startTourDivider.setGone(viewState.estate.multitourId == null)

        viewState.estate.multitourPreview?.let {
            Glide.with(tourScreen)
                .load(it)
                .into(tourScreen)
        }
        
        when(viewState.estate.type){
            FlatType.FLAT -> {
                flatTypeView.setGone(false)
                infoTabDivicder.setGone(false)
                explicationTab.setGone(false)
            }
            else -> {
                flatTypeView.setGone(true)
                infoTabDivicder.setGone(true)
                explicationTab.setGone(true)
            }
        }

    }

    private fun render(viewState: EstateScreenViewState.ShowEstateInfo) {

        (flatInfoRecyclerView.adapter as FlatInfoAdapter).setItems(viewState.info)

    }



    private fun render(viewState: EstateScreenViewState.LoadPlanTypes) {
        planTypesTabLayout.removeAllTabs()
        var i = 1

        viewState.types.forEach {
            try {
                val tab = planTypesTabLayout.newTab()
                    .setCustomView(R.layout.select_text_tab).setTag(it.planId)
                
                (tab.customView as MaterialTextView).text = "Вариант $i"
                planTypesTabLayout.addTab(tab)
                i++
            } catch (e: Exception) {
                Timber.e(e)
            }
        }
        planTypeSelected(viewState.types.first())
    }

    private fun planTypeSelected(plan:PlanPresetModel){
        wallsContainer.setGone(true)
        sizesContainer.setGone(true)
        electricContainer.setGone(true)
        furnContainer.setGone(true)
        furnitureDivider.setGone(true)
        wallsDivider.setGone(true)
        sizesDivider.setGone(true)
        electricityDivider.setGone(true)

        plan.features.forEach {
            when(it){
                is FeatureModel.Furniture -> {
                    furnContainer.setGone(false)
                    furnTitle.text = it.featureTitle
                    furnSwitcher.findViewById<SwitchMaterial>(R.id.switch1).isChecked =
                        it.switchedOn
                    furnitureDivider.setGone(false)
                }
                is FeatureModel.Walls ->{
                    wallsContainer.setGone(false)
                    wallsTitle.text = it.featureTitle
                    wallsSwitcher.findViewById<SwitchMaterial>(R.id.switch1).isChecked =
                        it.switchedOn
                    wallsDivider.setGone(false)
                }
                is FeatureModel.Sizes ->{
                    sizesContainer.setGone(false)
                    sizesTitle.text = it.featureTitle
                    sizesSwitcher.findViewById<SwitchMaterial>(R.id.switch1).isChecked =
                        it.switchedOn
                    sizesDivider.setGone(false)
                }
                is FeatureModel.Electric ->{
                    electricContainer.setGone(false)
                    electricTitle.text = it.featureTitle
                    electricSwitcher.findViewById<SwitchMaterial>(R.id.switch1).isChecked =
                        it.switchedOn
                    electricityDivider.setGone(false)
                }
            }
        }
    }


    private fun render(viewState: EstateScreenViewState.PlanTypeSelected) {
        planTypeSelected(viewState.item)
    }


    private fun render(viewState: EstateScreenViewState.LoadPlan) {
        planWebView.settings.javaScriptEnabled = true
        planWebView.clearCache(true)
        val uri = "file:///${Uri.parse(viewState.planBody).path}"
        planWebView.loadUrl(uri)
    }

        private fun getComponent() = DaggerEstateScreenComponent.factory()
            .create(RoomParkApplication.component,activity as RoomParkMainActivity
            ,args.getInt(SELECTED_ESTATE_ID_KEY))
        .inject(this)

    override fun getLayoutId(): Int = R.layout.flat_full_card_screen


}
