package com.biganto.visual.roompark.presentation.screen.feeds

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.FeedsInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

private const val FEED_PREVIEW_PAGE_SIZE = 10

class FeedsScreenPresenter @Inject constructor(
    private val interactor: FeedsInteractor
)
    : BigantoBasePresenter<FeedsScreen, FeedsScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        {e: ExceptionString -> FeedsScreenViewState.SomeError(e)}


    private val restoreModel = RestoreModel(
        arrayListOf(),
        arrayListOf(),
        arrayListOf(),
        mutableListOf()
    )


    override fun attachView(view: FeedsScreen) {
        Timber.d("AttachView ")
        super.attachView(view)
    }

    override fun detachView() {
        super.detachView()
        restoreStateObservable.accept(FeedsScreenViewState.RestoreView(restoreModel))
    }

    override fun bindIntents() {

        val fetchCams = interactor.fetchCams()
            .map <FeedsScreenViewState>{
                restoreModel.cams = it.items
                FeedsScreenViewState.CamsList(it.items)
            }
            .onErrorReturn (::parseError)

        val fetchAlbums = interactor.fetchAlbums()
            .map <FeedsScreenViewState>{
                Timber.d("got albums: $it")
                restoreModel.albums = it
                FeedsScreenViewState.AlbumsPages(it)
            }
            .onErrorReturn (::parseError)

        val fetchFeeds = interactor.fetchTopFeeds()
            .map <FeedsScreenViewState>{
                restoreModel.feeds = it.feeds
                FeedsScreenViewState.FeedsPages(it.feeds.toList())
            }
            .onErrorReturn (::parseError)

        val  getFeedArticlesPreview = intent(FeedsScreen::feedsTabSelected)
            .flatMap { interactor.fetchArticles(it) }
            .map<FeedsScreenViewState> {articlePage ->
                articlePage.articles
                    .asSequence()
                    .filter { !restoreModel.articles.contains(it) }
                    .toList()
                    .let {uniqueItems->restoreModel.articles.addAll(uniqueItems)}
                FeedsScreenViewState.GetFeedArticlesPreview(articlePage.articles.toList())
            }
            .onErrorReturn (::parseError)

        val  switchToAllArtiles = intent(FeedsScreen::onAllFeedArticles)
            .flatMap{ Observable.defer{Observable.just<FeedsScreenViewState>(
                FeedsScreenViewState.ToArticlesScreen(it.alias)) }}

        val  selectCamera = intent(FeedsScreen::onCameraSelected)
            .map<FeedsScreenViewState> {
                FeedsScreenViewState.ToWebCam(it.index)
            }

        val  selectArticle = intent(FeedsScreen::onArticleSelected)
            .map<FeedsScreenViewState> {
                FeedsScreenViewState.ToArticle(it.articleId)
            }

        val  selectAlbum = intent(FeedsScreen::onAlbumSelected)
            .map<FeedsScreenViewState> {
                FeedsScreenViewState.ToAlbum(it.albumId)
            }


        val  getNewArticlesPage = intent(FeedsScreen::requsetsNewArticles)
            .flatMap { interactor.fetchArticles(it.first.alias, FEED_PREVIEW_PAGE_SIZE,it.second) }
            .map<FeedsScreenViewState> {articlePage ->
                articlePage.articles
                    .asSequence()
                    .filter { !restoreModel.articles.contains(it) }
                    .toList()
                    .let {uniqueItems->restoreModel.articles.addAll(uniqueItems)}

                FeedsScreenViewState.ArticlesScrollPage(articlePage.articles.toList())
            }
            .onErrorReturn (::parseError)

        val state =

        fetchFeeds
            .mergeWith(getFeedArticlesPreview)
            .mergeWith(fetchAlbums)
            .mergeWith(fetchCams)
            .mergeWith(getNewArticlesPage)
            .mergeWith(switchToAllArtiles)
            .mergeWith(selectCamera)
            .mergeWith(selectAlbum)
            .mergeWith(selectArticle)
            .mergeWith(restoreStateObservable)
            .doOnError{ Timber.e(it)}
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(FeedsScreenViewState::class.java), FeedsScreen::render)
    }

}