package com.biganto.visual.roompark.presentation.screen.photo

import android.os.Bundle
import android.view.View
import android.view.WindowManager
import android.widget.ImageView
import androidx.core.os.bundleOf
import androidx.viewpager2.widget.ViewPager2
import butterknife.BindView
import butterknife.OnClick
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.conductor.dialogs.ChooseResolutionDialogController
import com.biganto.visual.roompark.conductor.dialogs.change_handler.DialogChangeHandler
import com.biganto.visual.roompark.presentation.screen.photo.util.PhotoPreviewSlider
import com.biganto.visual.roompark.presentation.screen.photo.util.PhotosAdapter
import com.bluelinelabs.conductor.RouterTransaction
import com.google.android.material.textview.MaterialTextView
import timber.log.Timber
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

const val SELECTED_PHOTO_KEY = "SELECTED_PHOTO_ID"

class PhotoScreenController :
    BigantoBaseController<PhotoScreenViewState
            , PhotoScreen
            , PhotoScreenPresenter>
   , PhotoScreen {

    constructor(args: Bundle):super(args)

    constructor(id: Int) : super(bundleOf(SELECTED_PHOTO_KEY to id))


    @BindView(R.id.photoTitle)
    lateinit var albumTitle: MaterialTextView

    @BindView(R.id.photo_frame)
    lateinit var photoViewPager: ViewPager2

    @BindView(R.id.photosPreviewSlider)
    lateinit var slider: PhotoPreviewSlider

    @BindView(R.id.choose_size_button)
    lateinit var chooseSizeButton : ImageView

    @OnClick(R.id.choose_size_button)
    fun onSwitchSize(){
        router.pushController(RouterTransaction.with(ChooseResolutionDialogController(ArrayList(
                (photoViewPager.adapter as PhotosAdapter)
                    .getItem(photoViewPager.currentItem).resolutionList
            )
        )).pushChangeHandler(DialogChangeHandler())
            .popChangeHandler(DialogChangeHandler())
        )
    }


    @OnClick(R.id.close_current_button)
    fun onCloseArticle(){
        handleBack()
    }

    override fun onAttach(view: View) {
//        tempSystemUiFlag = activity?.window?.decorView?.systemUiVisibility
        super.onAttach(view)
    }

    private var tempSystemUiFlag : Int? = View.SYSTEM_UI_FLAG_LIGHT_STATUS_BAR
    override fun onDetach(view: View) {
//        activity?.window?.decorView?.systemUiVisibility = tempSystemUiFlag?:View.SYSTEM_UI_FLAG_LAYOUT_STABLE
        super.onDetach(view)
    }

    private fun bindRecycler() {
        photoViewPager.isNestedScrollingEnabled = false
        photoViewPager.offscreenPageLimit = 1
        photoViewPager.adapter = PhotosAdapter()
        setVewPager()
    }

    private fun setVewPager(){
        val ap = PhotosAdapter()

        photoViewPager.adapter = ap
        photoViewPager.orientation = ViewPager2.ORIENTATION_HORIZONTAL
        photoViewPager.offscreenPageLimit = 2
        ap.notifyDataSetChanged()
        photoViewPager.invalidate()
        slider.setUpViewPager(photoViewPager)
    }

    override fun onViewBound(v: View) {
        toolBar.setToolbar()
        bottomNavigationController.hide()
        bindRecycler()

//        activity?.window?.decorView?.systemUiVisibility = View.SYSTEM_UI_FLAG_FULLSCREEN
        activity?.window?.addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN)
    }

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: PhotoScreenPresenter


//    @Inject
//    lateinit var snacky:ISnackBarProvider

    lateinit var rpActivity: RoomParkMainActivity

    fun getComponent() = DaggerPhotoScreenComponent.factory()
        .create(RoomParkApplication.component
            ,activity as RoomParkMainActivity
            ,args.getInt(SELECTED_PHOTO_KEY)).inject(this)

    override fun render(viewState: PhotoScreenViewState) {
        Timber.d("render: $viewState")
        when(viewState){
            is PhotoScreenViewState.Idle -> render(viewState)
            is PhotoScreenViewState.PhotoSelected -> render(viewState)
            is PhotoScreenViewState.PhotoFetched -> render(viewState)
            is PhotoScreenViewState.PhotoListLoaded -> render(viewState)
            is PhotoScreenViewState.AldumFetched-> render(viewState)
            is PhotoScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: PhotoScreenViewState.Idle){

    }

    private fun render(viewState: PhotoScreenViewState.SomeError) =
        showError(viewState.exception)

    private fun render(viewState: PhotoScreenViewState.PhotoListLoaded) {
        (photoViewPager.adapter as PhotosAdapter).setItems(
            viewState.list.asSequence().sortedBy{it.sort}.toList()
        )

        slider.visibility= View.VISIBLE
        slider.setUpViewPager(photoViewPager)

        photoViewPager.currentItem =
            (photoViewPager.adapter as PhotosAdapter).indexById(viewState.selectedId)
    }

    private fun render(viewState: PhotoScreenViewState.AldumFetched) {
        albumTitle.text = viewState.model.title
    }


    private fun render(viewState: PhotoScreenViewState.PhotoFetched){
        (photoViewPager.adapter as PhotosAdapter).setItems(arrayListOf(viewState.model))
    }

    private fun render(viewState: PhotoScreenViewState.PhotoSelected){
//        (photoViewPager.adapter as PhotoViewerAdapter).setItems(arrayListOf(viewState.model))
    }

    override fun handleBack(): Boolean {
        activity?.window?.clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN)
//        activity?.window?.decorView?.systemUiVisibility = tempSystemUiFlag?:View.SYSTEM_UI_FLAG_LAYOUT_STABLE
        router.popController(this)
        return true
    }

    override fun getLayoutId(): Int = R.layout.photo_view_screen

}