package com.biganto.visual.roompark.presentation.screen.splash

import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.SplashInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class SplashScreenPresenter @Inject constructor(
    private val acitivityContext: RoomParkMainActivity
    , private val interactor: SplashInteractor
//    ,var context:Context
)
    : BigantoBasePresenter<SplashScreen, SplashScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        {e: ExceptionString -> SplashScreenViewState.SomeError(e)}

    override fun bindIntents() {

        val state = restoreStateObservable
            .mergeWith(interactor.getAuth()
                .map {
                    if (it) SplashScreenViewState.ToHomeScreen()
                    else SplashScreenViewState.ToAuthScreen()
                }
                .delay (250, TimeUnit.MILLISECONDS)
            )
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
        subscribeViewState(state.cast(SplashScreenViewState::class.java), SplashScreen::render)
    }

}