package com.biganto.visual.roompark.presentation.screen.web_cam

import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.WebCamsInteractor
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class WebCamScreenPresenter @Inject constructor(
    private val interactor: WebCamsInteractor
    ,@Named(SELECTED_CAM_INDEX_KEY) private var selectedCamIndex:Int
)
    : BigantoBasePresenter<WebCamScreen, WebCamScreenViewState>() {

    override fun defaultErrorViewStateHandler() =
        {e: ExceptionString -> WebCamScreenViewState.SomeError(e) }

    override fun bindIntents() {

        val state = restoreStateObservable
            .mergeWith(interactor.fetchCams()
                .map {WebCamScreenViewState.WebCamsLIstLoaded(it.items.toList(),selectedCamIndex)}
            )
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
        subscribeViewState(state.cast(WebCamScreenViewState::class.java), WebCamScreen::render)
    }

}