package com.biganto.visual.roompark.util.extensions

import android.app.Activity
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.RectF
import android.net.Uri
import android.view.View
import timber.log.Timber
import java.io.File
import java.text.DecimalFormat
import java.text.SimpleDateFormat
import java.util.*


/**
 * Created by Vladislav Bogdashkin on 23.10.2019.
 */

private val dateFormatter = SimpleDateFormat("dd / MM / yyyy", Locale.getDefault())

val Date.formatToSimple: String
    get() = dateFormatter.format(this)

val Boolean?.asInt
    get() = if (this != null && this) 1 else 0

fun Int.toRubly() = String.format("%,d \u20BD",this).replace(',',' ')

fun Long.bytesToSize():String{
    if (this==-1L) return "~"
    var scale=0
    var rest:Long=this
    var least=0f
    while (rest>=1024) {
        least=(rest%1024).toFloat()
        rest /= 1024
        scale++
    }
    least/=102.4f
    return when(scale)
    {
        0->"$rest.${least.format(0)}B"
        1->"$rest.${least.format(0)}Kb"
        2->"$rest.${least.format(0)}Mb"
        3->"$rest.${least.format(0)}Gb"
        4->"$rest.${least.format(0)}Tb"
        else ->"$rest${least.format(0)}Bb" //BigantoByte
    }
}

fun Float.format(fracDigits: Int): String {
    val df = DecimalFormat()
    df.maximumFractionDigits = fracDigits
    return df.format(this)
}


fun View.setGone(isGone:Boolean){
    this.visibility = if (isGone) View.GONE else View.VISIBLE
}


fun Bitmap.scaleCenterCrop(viewHolder:View): Bitmap {
    Timber.d("go crpo")
    val sourceWidth = this.width
    val sourceHeight = this.height

    // Compute the scaling factors to fit the new height and width, respectively.
    // To cover the final image, the final scaling will be the bigger
    // of these two.
    val xScale = viewHolder.measuredWidth.toFloat() / sourceWidth
    val yScale = viewHolder.measuredHeight.toFloat() / sourceHeight
    val scale = xScale.coerceAtLeast(yScale)

    Timber.d(" left/top $xScale/$yScale")

    // Now get the displaySize of the source bitmap when scaled
    val scaledWidth = scale * sourceWidth
    val scaledHeight = scale * sourceHeight

    // Let's find out the upper left coordinates if the scaled bitmap
    // should be centered in the new displaySize give by the parameters
    val left = (viewHolder.measuredWidth - scaledWidth) / 2
    val top = (viewHolder.measuredHeight - scaledHeight) / 2

    Timber.d(" left/top $left/$top")

    // The target rectangle for the new, scaled version of the source bitmap will now
    // be
    val targetRect = RectF(left, top, left + scaledWidth, top + scaledHeight)

    // Finally, we create a new bitmap of the specified displaySize and draw our new,
    // scaled bitmap onto it.
    val dest = Bitmap.createBitmap(viewHolder.measuredWidth, viewHolder.measuredHeight, this.config)
    val canvas = Canvas(dest)

    Timber.d(" targetRect $targetRect")
    canvas.drawBitmap(this, null, targetRect, null)

    return dest
}

val File.folderSize get() =
    this.walkTopDown()
    .asSequence()
    .map {it.length()}
    .sum()


fun Activity.startUrl(url:String){
    val browserIntent = Intent(Intent.ACTION_VIEW, Uri.parse(url))
    this.startActivity(browserIntent)
}