package com.biganto.visual.roompark.util.view_utils.image_view

import android.content.Context
import android.graphics.Canvas
import android.graphics.Path
import android.graphics.Path.FillType
import android.graphics.RectF
import android.util.AttributeSet
import android.view.View
import android.widget.ImageView
import com.biganto.visual.roompark.R

/**
 * Created by Vladislav Bogdashkin on 23.05.2019.
 */
class RoundedImageView  @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : ImageView(context, attrs, defStyleAttr){

    private lateinit var mMaskPath: Path

    private val array =
        getContext().obtainStyledAttributes(attrs, R.styleable.RoundedImageView)

    private var mCornerRadius =
        array.getDimensionPixelSize(R.styleable.RoundedImageView_image_corner_radius,0)

    fun init() {
        this.setLayerType(View.LAYER_TYPE_HARDWARE, null)
        generateMaskPath(width, height)
        invalidate()
    }


    /**
     * Set the corner radius to use for the RoundedRectangle.
     *
     * @param Primitive int - The corner radius of the rounded rectangle.
     */
    fun setCornerRadius(cornerRadius: Int) {
        mCornerRadius = cornerRadius
        generateMaskPath(width, height)
        invalidate()
    }

    override fun onSizeChanged(w: Int, h: Int, oldW: Int, oldH: Int) {
        super.onSizeChanged(w, h, oldW, oldH)

        if (w != oldW || h != oldH) {
            generateMaskPath(w, h)
        }
    }

    private fun generateMaskPath(w: Int, h: Int) {
        mMaskPath = Path()
        mMaskPath.addRoundRect(RectF(
            0f
            , 0f
            , w.toFloat()
            , h.toFloat())
            , mCornerRadius.toFloat()
            , mCornerRadius.toFloat()
            , Path.Direction.CCW)
        mMaskPath.fillType = FillType.WINDING
    }

    val  alphaRect = RectF(RectF(0f, 0f, width.toFloat(), height.toFloat()))

    override fun onDraw(canvas: Canvas) {

        if (canvas.isOpaque) { // If canvas is opaque, make it transparent
            alphaRect.set(0f, 0f, width.toFloat(), height.toFloat())
            canvas.saveLayerAlpha(alphaRect,255)
        }
        canvas.clipPath(mMaskPath)
        super.onDraw(canvas)
    }
}