package com.biganto.visual.roompark.util.view_utils.status_progress_view

import android.content.Context
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.PointF
import android.util.AttributeSet
import android.view.View
import com.biganto.visual.roompark.R

/**
 * Created by Vladislav Bogdashkin on 19.09.2019.
 */
class StatusProgressCeil @JvmOverloads constructor(
    context: Context, attrs: AttributeSet? = null, defStyleAttr: Int = 0
) : View(context, attrs, defStyleAttr) {


    private val array =
        getContext().obtainStyledAttributes(attrs, R.styleable.StatusProgressCeil)

    private var direction:StatusProgressDirection =
        StatusProgressDirection.fromInt(array.getInt(R.styleable.StatusProgressCeil_direction,1))

    private var animateState:StatusProgressAnimationState =
        StatusProgressAnimationState.fromInt(array.getInt(R.styleable.StatusProgressCeil_anim_state,1))

    private var hasStart:Boolean =
        array.getBoolean(R.styleable.StatusProgressCeil_hasStart,false)
    private var hasEnd:Boolean =
        array.getBoolean(R.styleable.StatusProgressCeil_hasEnd,false)
    private var isEnable:Boolean =
        array.getBoolean(R.styleable.StatusProgressCeil_isEnable,false)
    private var nextEnable:Boolean =
        array.getBoolean(R.styleable.StatusProgressCeil_nextEnable,false)

    fun setHasStart(has:Boolean) { hasStart = has }
    fun setHasEnd(has:Boolean) { hasEnd = has }
    fun setIsEnabled(isEnable:Boolean) { this.isEnable = isEnable}
    fun setNext(nextisEnable:Boolean) { nextEnable = nextisEnable }
    fun setAnimState(state:StatusProgressAnimationState) { animateState = state }

    private val animationTimeMills:Long = 80L
    private var lastAnimateStateChanged:Long = 0L

    val ifAnimateEnd:Boolean get(){
        return (System.currentTimeMillis()-lastAnimateStateChanged) > animationTimeMills
    }



    private val fillEnableColor:Int = resources.getColor(R.color.colorCommonBackground)
    private val fillDisableColor:Int = resources.getColor(R.color.colorOpacityBackground)

    private val enablePaint:Paint = Paint()
    get(){
        field.style = Paint.Style.FILL_AND_STROKE
        field.color = fillEnableColor
        return field
    }

    private val disablePaint:Paint = Paint()
        get(){
            field.style = Paint.Style.FILL_AND_STROKE
            field.color = fillDisableColor
            return field
        }


    private val prorgressHalfW:Float
        get() { return when(direction){
            StatusProgressDirection.HORIZONTAL -> measuredHeight/16f
            StatusProgressDirection.VERTICAL -> measuredWidth/16f
        }}

    private val enableRadius:Float
    get() { return when(direction){
        StatusProgressDirection.HORIZONTAL -> measuredHeight/4f
        StatusProgressDirection.VERTICAL -> measuredWidth/4f
    }}


    private val crossRadius:Float
        get() { return when(direction){
            StatusProgressDirection.HORIZONTAL -> measuredHeight/2.4f
            StatusProgressDirection.VERTICAL -> measuredWidth/2.4f
        }}

    private val disableRadius:Float
        get() { return when(direction){
            StatusProgressDirection.HORIZONTAL -> measuredHeight/4f
            StatusProgressDirection.VERTICAL -> measuredWidth/4f
        }}


    //region startProgressRect
    private val progressStartLeft:Float
        get() {
            return when (direction) {
                    StatusProgressDirection.HORIZONTAL -> 0f
                    StatusProgressDirection.VERTICAL -> measuredWidth / 2f - prorgressHalfW
                }
        }
    private val progressStartTop:Float
        get() {
            return when (direction) {
                StatusProgressDirection.HORIZONTAL -> measuredHeight / 2f - prorgressHalfW
                StatusProgressDirection.VERTICAL -> 0f
            }
        }
    private val progressStartRight:Float
        get() {
            return when (direction) {
                StatusProgressDirection.HORIZONTAL -> measuredWidth/2f
                StatusProgressDirection.VERTICAL -> measuredWidth / 2f + prorgressHalfW
            }
        }
    private val progressStartBottom:Float
        get() {
            return when (direction) {
                StatusProgressDirection.HORIZONTAL -> measuredHeight / 2f + prorgressHalfW
                StatusProgressDirection.VERTICAL -> measuredHeight / 2f
            }
        }
    //endregion

    //region endProgressRect
    private val progressEndLeft:Float
        get() {
            return when (direction) {
                StatusProgressDirection.HORIZONTAL -> width/2f
                StatusProgressDirection.VERTICAL -> width / 2f - prorgressHalfW
            }
        }
    private val progressEndTop:Float
        get() {
            return when (direction) {
                StatusProgressDirection.HORIZONTAL -> height / 2f - prorgressHalfW
                StatusProgressDirection.VERTICAL -> height / 2f
            }
        }
    private val progressEndRight:Float
        get() {
            return when (direction) {
                StatusProgressDirection.HORIZONTAL -> width.toFloat()
                StatusProgressDirection.VERTICAL -> width / 2f + prorgressHalfW
            }
        }
    private val progressEndBottom:Float
        get() {
            return when (direction) {
                StatusProgressDirection.HORIZONTAL -> height / 2f + prorgressHalfW
                StatusProgressDirection.VERTICAL -> height.toFloat()
            }
        }
    //endregion

    private var enableDisableCrossRadius:Int = 0
    private val centerStatus:PointF
    get() {
        return when(direction){
            StatusProgressDirection.HORIZONTAL ->
                PointF(width/2f,height/2f)
            StatusProgressDirection.VERTICAL ->
                PointF(width/2f,height/2f)
        }
    }


    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)


        canvas?.drawColor(Color.TRANSPARENT)

        when(animateState){
            StatusProgressAnimationState.ENABLE -> drawEnable(canvas)
            StatusProgressAnimationState.DISABLE -> drawDisable(canvas)
            StatusProgressAnimationState.TOENABLE -> drawToEnable(canvas)
            StatusProgressAnimationState.TODISABLE -> drawToDisable(canvas)

        }


    }

    fun drawStartProgress(canvas: Canvas?){
        if (hasStart)
            canvas?.drawRect(
                progressStartLeft,
                progressStartTop,
                progressStartRight,
                progressStartBottom,
                if (isEnable) enablePaint else disablePaint
            )
    }

    fun drawEndProgress(canvas: Canvas?){
        if (hasEnd)
            canvas?.drawRect(
                progressEndLeft,
                progressEndTop,
                progressEndRight,
                progressEndBottom,
                if (nextEnable) enablePaint else disablePaint
            )
    }

    private fun drawEnable(canvas: Canvas?){
        drawStartProgress(canvas)
        drawEndProgress(canvas)
        canvas?.drawCircle(centerStatus.x,centerStatus.y
            ,if (nextEnable) enableRadius else crossRadius
            ,enablePaint)
    }

    private fun drawDisable(canvas: Canvas?){
        drawStartProgress(canvas)
        drawEndProgress(canvas)
        canvas?.drawCircle(centerStatus.x,centerStatus.y,enableRadius,disablePaint)
    }

    private fun drawToEnable(canvas: Canvas?){
        drawStartProgress(canvas)
        drawEndProgress(canvas)
        canvas?.drawCircle(centerStatus.x,centerStatus.y,enableRadius,enablePaint)
    }

    private fun drawToDisable(canvas: Canvas?){
        drawStartProgress(canvas)
        drawEndProgress(canvas)
        canvas?.drawCircle(centerStatus.x,centerStatus.y,enableRadius,disablePaint)
    }

}


enum class StatusProgressDirection(val dir:Int){
    HORIZONTAL(1),
    VERTICAL(2);

    companion object {
        fun fromInt(value: Int) = values().first { it.dir == value }
    }
}


enum class StatusProgressAnimationState(val state:Int){
    ENABLE(1),
    DISABLE(2),
    TOENABLE(3),
    TODISABLE(4);

    companion object {
        fun fromInt(value: Int) = StatusProgressAnimationState.values().first { it.state == value }
    }
}
